@@ -104,7 +104,68 @@ level::level(const std::string& level_cfg)
 	const int start_time = SDL_GetTicks();
 
 	wml::const_node_ptr node = load_level_wml(level_cfg);
+	wml::const_node_ptr player_save_node;
 	ASSERT_LOG(node.get() != NULL, "LOAD LEVEL WML FOR " << level_cfg << " FAILED");
+	if(node->has_attr("id")) {
+		id_ = node->attr("id");
+	}
+
+	if(preferences::load_compiled() && (level_cfg == "save.cfg" || level_cfg == "autosave.cfg")) {
+		if(preferences::version() != node->attr("version").str()) {
+			std::cerr << "DIFFERENT VERSION LEVEL\n";
+			FOREACH_WML_CHILD(obj_node, node, "character") {
+				if(wml::get_bool(obj_node, "is_human", false)) {
+					player_save_node = obj_node;
+					break;
+				}
+			}
+
+			wml::const_node_ptr n = node;
+			if(node->has_attr("id")) {
+				n = load_level_wml(node->attr("id"));
+			} else {
+				//this save was made before we saved level ID's. The best
+				//we can do is get a level with a matching title.
+				std::vector<std::string> files;
+				sys::get_files_in_dir("data/compiled/level/", &files);
+				foreach(const std::string& file, files) {
+					if(file == "save.cfg" || file == "autosave.cfg") {
+						continue;
+					}
+
+					wml::const_node_ptr lvl_info = load_level_wml(file);
+					if(lvl_info->attr("title").str() == node->attr("title").str()) {
+						if(lvl_info->attr("dimensions").str() == node->attr("dimensions").str()) {
+							//the dimensions match so we're sure this is
+							//the right one.
+							n = lvl_info;
+							break;
+						}
+
+						//tenatively guess this is the level, but keep
+						//searching in case one with an exact dimensional
+						//match is found.
+						n = lvl_info;
+					}
+				}
+			}
+			//in case the player had swallowed an object, copy it
+			//over from the savegame.
+			wml::node::const_child_iterator i1 = node->begin_child("serialized_objects");
+			wml::node::const_child_iterator i2 = node->end_child("serialized_objects");
+			wml::node::const_child_iterator i3 = n->begin_child("serialized_objects");
+			while(i1 != i2) {
+				wml::node::const_child_iterator j1 = (i1->second)->begin_child("character");
+				wml::node::const_child_iterator j2 = (i1->second)->end_child("character");
+				while(j1 != j2) {
+					(i3->second)->add_child(j1->second);
+					++j1;
+				}
+				++i1;
+			}
+			node = n;
+		}
+	}
 
 	if(wml::get_bool(node, "dark", false)) {
 		dark_ = true;
@@ -238,10 +299,18 @@ level::level(const std::string& level_cfg)
 	wml::node::const_child_iterator c1 = node->begin_child("character");
 	wml::node::const_child_iterator c2 = node->end_child("character");
 	for(; c1 != c2; ++c1) {
+		if(player_save_node.get() != NULL && wml::get_bool(c1->second, "is_human", false)) {
+			continue;
+		}
+
 		wml_chars_.push_back(c1->second);
 		continue;
 	}
 
+	if(player_save_node.get() != NULL) {
+		wml_chars_.push_back(player_save_node);
+	}
+
 	wml::const_node_ptr serialized_objects = node->get_child("serialized_objects");
 	if(serialized_objects.get() != NULL) {
 		wml_chars_.push_back(serialized_objects);
@@ -815,6 +884,8 @@ wml::node_ptr level::write() const
 	game_logic::wml_formula_callable_serialization_scope serialization_scope;
 
 	wml::node_ptr res(new wml::node("level"));
+	res->set_attr("id", id_);
+	res->set_attr("version", preferences::version());
 	res->set_attr("title", title_);
 	res->set_attr("music", music_);
 	res->set_attr("segment_width", formatter() << segment_width_);
@@ -3312,6 +3383,7 @@ void level::add_sub_level(const std::string& lvl, int xoffset, int yoffset, bool
 
 			relocate_object(c, c->x() + itor->second.xoffset, c->y() + itor->second.yoffset);
 			add_character(c);
+			c->handle_event(OBJECT_EVENT_START_LEVEL);
 
 			itor->second.objects.push_back(c);
 		}
